############################### This program is free software; you can redistribute it and/or modify
############################### it under the terms of the GNU General Public License as published by
############################### the Free Software Foundation; either version 2 of the License, or
############################### (at your option) any later version.
############################### 
############################### This program is distributed in the hope that it will be useful,
############################### but WITHOUT ANY WARRANTY; without even the implied warranty of
############################### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
############################### GNU General Public License for more details.
############################### 
############################### You should have received a copy of the GNU General Public License
############################### along with this program; if not, write to the Free Software
############################### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
###############################
###############################
###############################
############################### Copyright 2009 Damon P. Little

USE mysql;

INSERT IGNORE INTO user SET Host = 'localhost', User = 'realBOLD', Password = password('realBOLD'), Select_priv = 'Y', Insert_priv = 'Y', Update_priv = 'Y', Delete_priv = 'Y', Create_tmp_table_priv = 'Y'
ON DUPLICATE KEY UPDATE Host = 'localhost', User = 'realBOLD', Password = password('realBOLD'), Select_priv = 'Y', Insert_priv = 'Y', Update_priv = 'Y', Delete_priv = 'Y', Create_tmp_table_priv = 'Y';

FLUSH PRIVILEGES;

DROP DATABASE IF EXISTS `markers`;
CREATE DATABASE `markers`;

USE markers;

DROP TABLE IF EXISTS taxa;
CREATE TABLE `taxa` (
  `taxonID` SMALLINT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
  `clade` CHAR(24) NOT NULL,
  `genus` CHAR(24) NOT NULL,
  `species` CHAR(24) NOT NULL,
  PRIMARY KEY  (`taxonID`),
  KEY `cladeIndex` (`clade`),
  KEY `genusIndex` (`genus`),
  UNIQUE KEY `nomen` (`genus`,`species`)
) ENGINE=MyISAM DEFAULT CHARSET=ascii;

DROP TABLE IF EXISTS specimens;
CREATE TABLE `specimens` (
  `specimenID` SMALLINT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
  `taxonID` SMALLINT(11) UNSIGNED NOT NULL,
  `collection` CHAR(128) NOT NULL UNIQUE,
  `reference` CHAR(32),
  PRIMARY KEY  (`specimenID`),
  KEY `taxonIDIndex` (`taxonID`)
) ENGINE=MyISAM DEFAULT CHARSET=ascii;

DROP TABLE IF EXISTS loci;
CREATE TABLE `loci` (
  `locusID` TINYINT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
  `locus` CHAR(4) NOT NULL UNIQUE,
  `locusLong` CHAR(24) NOT NULL UNIQUE,
  PRIMARY KEY  (`locusID`),
  KEY `locusIndex` (`locus`)
) ENGINE=MyISAM DEFAULT CHARSET=ascii;

DROP TABLE IF EXISTS sequences;
CREATE TABLE `sequences` (
  `sequenceID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
  `locusID` TINYINT(11) UNSIGNED NOT NULL,
  `specimenID` SMALLINT(11) UNSIGNED NOT NULL,
  `score` SMALLINT(11) UNSIGNED DEFAULT NULL,
  `sequence0ID` INT(11) UNSIGNED DEFAULT NULL,
  `sequence1ID` INT(11) UNSIGNED DEFAULT NULL,
  PRIMARY KEY  (`sequenceID`),
  KEY `locusIndex` (`locusID`),
  KEY `specimenIndex` (`specimenID`),
  UNIQUE KEY `locusSpecimenIndex` (`locusID`, `specimenID`),
  KEY `sequence0IDIndex` (`sequence0ID`),
  KEY `sequence1IDIndex` (`sequence1ID`),
  UNIQUE KEY `pairsIndex` (`sequence0ID`, `sequence1ID`)
) ENGINE=MyISAM DEFAULT CHARSET=ascii;

DROP TABLE IF EXISTS bases;
CREATE TABLE `bases` (
  `basesID` SMALLINT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
  `sequenceID` INT(11) UNSIGNED NOT NULL,
  `accession` CHAR(12) DEFAULT NULL,
  `sequence` TEXT,
  PRIMARY KEY (`basesID`),
  UNIQUE KEY `sequenceIDIndex` (`sequenceID`)
) ENGINE=MyISAM DEFAULT CHARSET=ascii;

DROP TABLE IF EXISTS distances;
CREATE TABLE `distances` (
  `distanceID` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
  `sequence0ID` INT(11) UNSIGNED NOT NULL,
  `sequence1ID` INT(11) UNSIGNED NOT NULL,
  `overlapping` ENUM('no', 'yes') DEFAULT 'no',
  `p` MEDIUMINT(11) UNSIGNED NOT NULL,
  `pDifference` SMALLINT(11) UNSIGNED NOT NULL,
  `pBases` SMALLINT(11) UNSIGNED NOT NULL,
  PRIMARY KEY  (`distanceID`),
  KEY `sequence0IDIndex` (`sequence0ID`),
  KEY `sequence1IDIndex` (`sequence1ID`),
  UNIQUE KEY `pairsIndex` (`sequence0ID`, `sequence1ID`),
  KEY `pIndex` (`p`)
) ENGINE=MyISAM DEFAULT CHARSET=ascii;

DROP TABLE IF EXISTS m;
DROP TABLE IF EXISTS r;
DROP TABLE IF EXISTS mr;

CREATE TABLE m LIKE distances;
CREATE TABLE r LIKE distances;
CREATE TABLE mr LIKE distances;
