#!/usr/bin/perl

############################### This program is free software; you can redistribute it and/or modify
############################### it under the terms of the GNU General Public License as published by
############################### the Free Software Foundation; either version 2 of the License, or
############################### (at your option) any later version.
############################### 
############################### This program is distributed in the hope that it will be useful,
############################### but WITHOUT ANY WARRANTY; without even the implied warranty of
############################### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
############################### GNU General Public License for more details.
############################### 
############################### You should have received a copy of the GNU General Public License
############################### along with this program; if not, write to the Free Software
############################### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
###############################
###############################
###############################
############################### Copyright 2009 Damon P. Little



############################### DETERMINE OPTIONS AND ANALYSIS TYPE
$i = ();
$l = ();
$go = 0;



for($k = 0; $k <= $#ARGV; $k++){
	if($ARGV[$k] eq "-i"){
		$i = $ARGV[$k+1];
		if(-e "$i"){
			$go++;
			}
		}
	if($ARGV[$k] eq "-l"){
		$l = $ARGV[$k+1];
		$l =~ tr/[a-z][A-Z][0-9]//cd;
		if(length($l)){
			$go++;
			}
		}
	}



if($go == 2){ ############################### INPUT GIVEN
	
	
	
	############################### DATABASE CONNECTION
	use DBI;
	my $database = "markers";
	my $host = "localhost";
	my $id = "realBOLD";
	my $password = "realBOLD";
	my $sql = DBI->connect("DBI:mysql:$database:$host",$id,$password);
	if(!defined $sql) { 
		die "Cannot connect: $DBI::errstr \n";
		}
	
	
	
	############################### LOCUS
	$query = "INSERT IGNORE INTO loci SET locus = ?, locusLong = ?";
	$sth = $sql->prepare($query) || die "cannot prepare $statement: $sql->errstr\n";
	$rv = $sth->execute($l, $l) || die "cannot execute the query: $sth->errstr";
	
	$query = "SELECT locusID FROM loci WHERE locus = ?";
	$sth = $sql->prepare($query) || die "cannot prepare $statement: $sql->errstr\n";
	$rv = $sth->execute($l) || die "cannot execute the query: $sth->errstr";
	$loci = $sth->fetchall_arrayref() || die "$sth->errstr\n";
	$L = $loci->[0][0];
	
	
	
	############################### READ AND UPLOAD SEQUENCES
	open(INFILE, "$i") || die("Could not open $i!");
	$count = 0;
	$seq = ();
	$name =();
	while($line = <INFILE>){
		chomp($line);
		if(length($line)){
			if($line =~ m/^>/){ ############################### IS NAME
				if(length($name) && length($seq)){
					$count += upload($seq, $name, $L);
					}
					$seq = ();
					$name = $line;
				} else { ############################### IS NOT NAME
					$seq .= $line;
					}
			}
		}
	close(INFILE);
	$count += upload($seq, $name, $L);
	print("$count records uploaded\n\n");
	
	
	
	############################### UPLOAD
	sub upload ($$$){
		my $seq = $_[0];
		my @name = $_[1];
		my $locus = $_[2];

		$seq = uc($seq);
		$seq =~ tr/ACGTNVDBHWMRKSY//cd;
		my $score = 4 * ($seq =~ tr/ATGC/ATGC/);
		$score += 2 * ($seq =~ tr/WMRKSY/WMRKSY/);
		$score += 1 * ($seq =~ tr/VDBH/VDBH/);
				
		$name =~ tr/[a-z][A-Z][0-9]\_//cd;
		my @nameBits = split(/\_/, $name);
		
		if(length($name) && length($seq)){
			my $query = "INSERT IGNORE INTO specimens SET collection = ?, taxonID = (SELECT taxonID FROM taxa WHERE genus = ? AND species = ?)";
			$sth = $sql->prepare($query) || die "cannot prepare $statement: $sql->errstr\n";
			$rv = $sth->execute($name, $nameBits[0], $nameBits[1]) || die "cannot execute the query: $sth->errstr";
			
			my $query = "INSERT INTO sequences SET locusID = ?, specimenID = (SELECT specimenID FROM specimens WHERE collection = ?), score = ?
				     ON DUPLICATE KEY UPDATE score = ?";
			$sth = $sql->prepare($query) || die "cannot prepare $statement: $sql->errstr\n";
			$rv = $sth->execute($locus, $name, $score, $score) || die "cannot execute the query: $sth->errstr";
			
			my $query = "INSERT INTO bases SET sequenceID = (SELECT sequenceID FROM sequences, specimens WHERE sequences.specimenID = specimens.specimenID AND locusID = ? AND collection = ? AND score = ?), sequence = ?
				     ON DUPLICATE KEY UPDATE sequence = ?";
			$sth = $sql->prepare($query) || die "cannot prepare $statement: $sql->errstr\n";
			$rv = $sth->execute($locus, $name, $score, $seq, $seq) || die "cannot execute the query: $sth->errstr";

			return(1);
			}
		}
	
	
	
	############################### CLEAN UP
	$rc = $sth->finish;
	$sql->disconnect;
	
	
	
	} else { ############################### INPUT OPTIONS NOT GIVEN
		print("\nA PERL script for loading FASTA formated sequences into\n");
		print("the tables `loci', `sequences`, and`specimens`.\n");
		print("Names should be in the form Genus_species_collection.\n\n");
		print("USAGE: sequences.pl -i in-file.fasta -l locus\n\n");
		}
		
		
		
exit;
